"""
Uses pisa from http://www.htmltopdf.org to do its handy work

"""
import logging
import tempfile
import shutil
import codecs
from StringIO import StringIO

from os.path import join, abspath, basename
from os import sep, mkdir

from Products.PortalTransforms.interfaces import itransform

from Products.Plone2Pdf.config import DEBUG, PROJECTNAME
from Products.Plone2Pdf.manipulation import handleImages, handleCSS, addLinkList

from sx.pisa3.pisa_document import pisaDocument

logger = logging.getLogger(PROJECTNAME)

class html_to_pdf:
    __implements__ = itransform
    
    __version__ = '2008-01-15.01'

    __name__ = "html_to_pdf"

    inputs   = ('text/html',)
    output  = 'application/pdf'
    output_encoding = 'utf-8'

    def __init__(self, name=None, **kwargs):
        if name:
            self.__name__ = name  

    def name(self):
        return self.__name__

    def initialize_tmpdir(self, data, **kwargs):
        """create a temporary directory, copy input in a file there
        return the path of the tmp dir and of the input file
        """
        tmpdir = tempfile.mktemp()
        mkdir(tmpdir)
        filename = kwargs.get("filename", '')
        fullname = join(tmpdir, basename(filename))
        filedest = codecs.open(fullname , "wb", encoding=self.output_encoding).write(data)
        return tmpdir, fullname

    def cleanDir(self, tmpdir):
        shutil.rmtree(tmpdir)    

    def convert(self, data, cache, **kwargs):

        input_data = data.viewPdf()

        tmpdir, fullname = self.initialize_tmpdir(input_data, filename='orig.html')

        input_data = handleCSS(data, input_data, tmpdir)
        input_data = handleImages(data, input_data, tmpdir)

        if kwargs.get('add_link_list',None):
           input_data = addLinkList(input_data)

        src = '%s%sfixed.html' % (tmpdir,sep)

        filedest = open(src , "wb").write(input_data)

        pdf = self.pdfConvert(src, tmpdir)

        self.cleanDir(tmpdir)

        cache.setData(pdf)

        return cache


    def pdfConvert(self, src, tmpdir):

        wpath = abspath(src)
        try:
            fsrc = open(src, 'rb')
            fdest = StringIO()
           
            res = pisaDocument(
                fsrc,
                fdest,
                debug = DEBUG,
                path = wpath,
                format = 'pdf',
                link_callback = None,
                default_css = None,
                xhtml = None,
                )
        
            pdf = fdest.getvalue()
           
        except:
            msg = u'pdf creation failed'
            logger.error(msg)
            return ''
        
        return pdf


def register():
    return html_to_pdf()
